import types
from version import __version__
from .model import Model
from .collection import Collection
from .spec.jsonschema import to_jsonschema


class CellarDoor(object):
	
	def __init__(self, collections=(), authenticators=(), storage=None):
		if type(collections) == types.ModuleType:
			collection_classes = []
			for k,v in collections.__dict__.items():
				try:
					if issubclass(v, Collection) and v != Collection:
						collection_classes.append(v)
				except TypeError:
					pass
		else:
			collection_classes = collections
		entities = set()
		collections_by_class_name = {}
		
		for collection_cls in collection_classes:
			entities.add(collection_cls.entity)
			collection = collection_cls(storage)
			collections_by_class_name[collection_cls.__name__] = collection
			setattr(self, collection_cls.plural_name, collection)
			
		self.model = Model(storage, entities)
		
		for collection in collections_by_class_name.values():
			new_links = {}
			if collection.links:
				for k, v in collection.links.items():
					if not isinstance(v, basestring):
						v = v.__name__
					referenced_collection = collections_by_class_name.get(v)
					new_links[k] = referenced_collection
			collection.links = new_links
		
		self.collections = collections_by_class_name.values()
		self.entities = entities
			
			
	def schema(self, base_url):
		return to_jsonschema(self, base_url)


import msgpack
from datetime import datetime
from . import Serializer


def default_handler(obj):
	try:
		iterable = iter(obj)
	except TypeError:
		pass
	else:
		return list(iterable)
	
	if isinstance(obj, datetime):
		return obj.isoformat()
	
	raise ValueError, "Can't pack object of type %s" % type(obj).__name__
	


class MsgPackSerializer(Serializer):
	
	mimetype = 'application/x-msgpack'
	
	def serialize(self, obj):
		return msgpack.packb(obj, default=default_handler)
		
		
	def unserialize(self, stream):
		return msgpack.unpack(stream)


class View(object):
	
	serializers = None
	
	def get_collection_response(self, req, objs):
		raise NotImplementedError
		
		
	def get_individual_response(self, req, obj):
		raise NotImplementedError
		
		
	def serialize(self, req, obj):
		content_type, serializer = self.get_serializer(req)
		return content_type, serializer.serialize(obj)
		
		
	def get_serializer(self, req):
		return self.choose(req, self.serializers)
		
		
	@classmethod
	def choose(cls, req, views):
		raw_accept = req.get_header('Accept')
		if raw_accept:
			accept = raw_accept.split(';')[0].split(',')
			for a in accept:
				for k,v in views:
					if a == k:
						return k, v
		return views[0]
		
		
from minimal import MinimalView


import io
import os.path
from setuptools import setup, find_packages

execfile(os.path.join('cellardoor', 'version.py'))

setup(
	name='cellardoor',
	version=__version__,
	description='Create ReST APIs how you\'ve always wanted.',
	long_description=io.open('README.rst', mode='r', encoding='utf-8').read(),
	classifiers=[
        'Development Status :: 1 - Planning',
        'Environment :: Web Environment',
        'Natural Language :: English',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: MacOS :: MacOS X',
        'Operating System :: Microsoft :: Windows',
        'Operating System :: POSIX',
        'Topic :: Internet :: WWW/HTTP :: WSGI',
        'Topic :: Software Development :: Libraries :: Application Frameworks',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7'
    ],
    keywords='wsgi web api framework rest http cloud',
    author='Elisha Fitch-Cook',
    author_email='elisha@cooper.com',
    url='http://github.com/cooper-software/cellardoor',
    license='MIT',
    packages=find_packages(exclude=['tests']),
    install_requires=['falcon'],
    extras_require={
        'mongo_storage': ['pymongo'],
        'timelib_parsing': ['timelib'],
        'dateutil_parsing': ['python-dateutil'],
        'msgpack': ['msgpack-python']
    },
    tests_require=[
        'mock',
        'nose', 
        'timelib', 
        'python-dateutil', 
        'msgpack-python', 
        'pymongo'],
    test_suite='nose.collector'
)


class EventManager(object):
	
	def __init__(self, *event_names):
		self.listeners = {
			'pre': dict([(k, []) for k in event_names]), 
			'post': dict([(k, []) for k in event_names])
		}
		
		
	def pre(self, event_name, fn):
		self.add('pre', event_name, fn)
		
		
	def post(self, event_name, fn):
		self.add('post', event_name, fn)
		
		
	def trigger_pre(self, event_name, *args, **kwargs):
		self.trigger('pre', event_name, *args, **kwargs)
		
		
	def trigger_post(self, event_name, *args, **kwargs):
		self.trigger('post', event_name, *args, **kwargs)
		
		
	def add(self, when, event_name, fn):
		self.listeners[when][event_name].append(fn)
		
		
	def trigger(self, when, event_name, *args, **kwargs):
		fns = self.listeners[when][event_name]
		for fn in fns:
			fn(*args, **kwargs)